@@ -29,31 +29,36 @@ from numpy.lib.function_base import iterable
 from matplotlib.backends.backend_agg import FigureCanvasAgg
 from .array_processing import get_Several_MinMax_Array
 
-
 popular_image_suffixes = ['png', 'jpg', 'jpeg', 'bmp']
 
 
-
-def erode(mat, iterations=1, kernel_size=3):
+def erode(mat, iterations=1, kernel_size=3, kernel=None):
     """ erode 2D binary matrix by one pixel """
-    assert isinstance(mat, np.ndarray) and mat.dtype == np.bool
-    kernel = np.ones((kernel_size, kernel_size), np.uint8)
+    assert isinstance(mat, np.ndarray)
+    if kernel is None:
+        kernel = np.ones((kernel_size, kernel_size), np.uint8)
+    else:
+        kernel = np.array(kernel, np.uint8)
     mat_eroded = cv.erode(mat.astype(np.uint8), kernel, iterations=iterations)
     return mat_eroded > 0
 
-
-def dilate(mat, iterations=1, kernel_size=3):
+def dilate(mat, iterations=1, kernel_size=3, kernel=None):
     """ dilate 2D binary matrix by one pixel """
-    assert isinstance(mat, np.ndarray) and mat.dtype == np.bool
-    kernel = np.ones((kernel_size, kernel_size), np.uint8)
+    assert isinstance(mat, np.ndarray)
+    if kernel is None:
+        kernel = np.ones((kernel_size, kernel_size), np.uint8)
+    else:
+        kernel = np.array(kernel, np.uint8)
     mat_dilated = cv.dilate(mat.astype(np.uint8), kernel, iterations=iterations)
     return mat_dilated > 0
 
-
-def open_op(mat, iterations=1, kernel_size=3):
+def open(mat, iterations=1, kernel_size=3, kernel=None):
     """ dilate 2D binary matrix by one pixel """
-    assert isinstance(mat, np.ndarray) and mat.dtype == np.bool
-    kernel = np.ones((kernel_size, kernel_size), np.uint8)
+    assert isinstance(mat, np.ndarray)
+    if kernel is None:
+        kernel = np.ones((kernel_size, kernel_size), np.uint8)
+    else:
+        kernel = np.array(kernel, np.uint8)
     mat_dilated = cv.erode(mat.astype(np.uint8), kernel, iterations=iterations)
     mat_dilated = cv.dilate(mat_dilated.astype(np.uint8), kernel, iterations=iterations)
     return mat_dilated > 0
@@ -134,8 +139,8 @@ def center_crop(input_image, width, height):
     return crop_data_around_boxes(input_image, crop_box)
 
 
-def image_resize(img_source, shape=None, factor=None, unique_check=False, interpolation=None):
-    if str(img_source.dtype).lower() != 'uint8':
+def image_resize(img_source, shape=None, factor=None, unique_check=False, interpolation=None, uint8=True):
+    if str(img_source.dtype).lower() != 'uint8' and uint8:
         print(f"Waring from mtutils image_resize: input source type {str(img_source.dtype)} is not uint8, we've made a transfer automatically.")
         img_source = img_source.astype('uint8')
     image_H, image_W = img_source.shape[:2]
@@ -678,7 +683,91 @@ def plt_image_show(*image, window_name='', array_res=False, full_screen=True, cm
 
 PIS = plt_image_show
 
+def scatter3d(xyz_data_list, label_list, xyz_label=None, title='3d Scatter plot'):
+    if not xyz_data_list:
+        return
+    xyz_data = np.array(xyz_data_list)
+    assert xyz_data.ndim == 2
+    assert xyz_data.shape[1] == 3
 
+    label_set = set(label_list)
+    label_2_color_dict = dict()
+    label_data_dict = dict()
+    for index, label in enumerate(label_set):
+        color = [np.random.rand(), np.random.rand(), np.random.rand()]
+        label_2_color_dict[label] = color
+        info = dict(
+            x=[],
+            y=[],
+            z=[]
+        )
+        label_data_dict[label] = info
+
+    for data, label in zip(xyz_data_list, label_list):
+        x, y, z = data
+        label_data_dict[label]['x'].append(x)
+        label_data_dict[label]['y'].append(y)
+        label_data_dict[label]['z'].append(z)
+
+    fig = plt.figure()
+    ax = plt.axes(projection='3d')
+    for label, data_set in label_data_dict.items():
+        ax.scatter3D(np.array(data_set['x']), np.array(data_set['y']), np.array(data_set['z']), c=[label_2_color_dict[label]] * len(data_set['x']), label=label)
+
+    ax.set_title(title)
+    try:
+        ax.set_xlabel(xyz_label[0])
+        ax.set_ylabel(xyz_label[1])
+        ax.set_zlabel(xyz_label[2])
+    except:
+        ax.set_xlabel('X')
+        ax.set_ylabel('Y')
+        ax.set_zlabel('Z')
+
+    plt.legend()
+    plt.show()
+    pass
+
+
+def scatter2d(xy_data_list, label_list, xy_label=None, title='2d Scatter plot'):
+    if not xy_data_list:
+        return
+    xyz_data = np.array(xy_data_list)
+    assert xyz_data.ndim == 2
+    assert xyz_data.shape[1] == 2
+
+    label_set = set(label_list)
+    label_2_color_dict = dict()
+    label_data_dict = dict()
+    for index, label in enumerate(label_set):
+        color = [np.random.rand(), np.random.rand(), np.random.rand()]
+        label_2_color_dict[label] = color
+        info = dict(
+            x=[],
+            y=[]
+        )
+        label_data_dict[label] = info
+
+    for data, label in zip(xy_data_list, label_list):
+        x, y = data
+        label_data_dict[label]['x'].append(x)
+        label_data_dict[label]['y'].append(y)
+
+    fig = plt.figure()
+    ax = plt.axes()
+    for label, data_set in label_data_dict.items():
+        ax.scatter(np.array(data_set['x']), np.array(data_set['y']), c=[label_2_color_dict[label]] * len(data_set['x']), label=label)
+
+    ax.set_title(title)
+    try:
+        ax.set_xlabel(xy_label[0])
+        ax.set_ylabel(xy_label[1])
+    except:
+        ax.set_xlabel('X')
+        ax.set_ylabel('Y')
+
+    plt.legend()
+    plt.show()
 
 def make_histgram(data_array_list, min_v=None, max_v=None, bin_num=None, gap=None, style_str_list=None, array_res=False):
     color_list=['', 'y', 'g', 'r', 'c', 'm', 'b', 'k', 'w']
@@ -783,6 +872,7 @@ def crop_data_around_boxes(image, crop_box, cut_box_back=False):
     """make a image crop from a image safely"""
 
     ndim = image.ndim
+    data_type = image.dtype
     height, width = image.shape[:2]
 
     crop_box = np.array(crop_box).astype('int32').tolist()
@@ -809,9 +899,9 @@ def crop_data_around_boxes(image, crop_box, cut_box_back=False):
 
         # out of boundary
         if ndim == 3:
-            crop_ori_temp = np.zeros([crop_height, crop_width, 3], dtype='uint8')
+            crop_ori_temp = np.zeros([crop_height, crop_width, 3], dtype=data_type)
         elif ndim == 2:
-            crop_ori_temp = np.zeros([crop_height, crop_width], dtype='uint8')
+            crop_ori_temp = np.zeros([crop_height, crop_width], dtype=data_type)
         else:
             raise RuntimeError(f"error image shape {image.shape} ndim {ndim}")
 
@@ -889,6 +979,61 @@ def zero_padding(in_array, padding_size_1, padding_size_2, padding_size_3=None,
     return padded_array
 
 
+def image_border_move(img, left, top=None, right=None, bottom=None, border_type='constant', value=0):
+    """
+    adjust image border reltively
+    Args:
+        img (np.array): image to be processed
+        left (int): left value
+        top (int): top value
+        right (int): right value
+        bottom (int): bottom value
+        border_type (str, optional): type for additional border, should be one of ['constant', 'replicate', 'reflect']. Defaults to 'constant'.
+        value (int, optional): if constant border is set, it means the padding pixel value. Defaults to 0.
+    """
+    if top is None:
+        top = left
+    if right is None:
+        right = top
+    if bottom is None:
+        bottom = right
+
+    assert border_type in ['constant', 'replicate', 'reflect']
+    left, top, right, bottom = vvd_round([left, top, right, bottom])
+    ori_H, ori_W = img.shape[:2]
+    assert -left < ori_W + right
+    assert -top < bottom + ori_H
+
+    move_data = np.array([left, top, right, bottom])
+
+    # padding
+    padding_data = np.clip(move_data, 0, None)
+    if padding_data.max() > 0:
+        assert len(padding_data) == 4
+        if border_type == 'constant':
+            border_type_index = cv2.BORDER_CONSTANT
+        elif border_type == 'replicate':
+            border_type_index = cv2.BORDER_REPLICATE
+        elif border_type == 'reflect':
+            border_type_index = cv2.BORDER_REFLECT_101
+        else:
+            raise RuntimeError(f"unknown border type {border_type}.")
+        padded_img = cv2.copyMakeBorder(img, padding_data[1], padding_data[3], padding_data[0], padding_data[2], borderType=border_type_index, value=value)
+    else:
+        padded_img = img
+
+    # croping
+    H, W = padded_img.shape[:2]
+    crop_data = np.clip(move_data, None, 0)
+    if crop_data.min() < 0:
+        bbox = [0 - crop_data[0], 0 - crop_data[1], W + crop_data[2], H + crop_data[3]]
+        crop_img = crop_data_around_boxes(padded_img, bbox)
+    else:
+        crop_img = padded_img
+
+    return crop_img
+
+
 def polar_move(polar_image, source_center_phase, target_center_phase):
     """[height of polar_image is the origin circle side]
 
@@ -1020,7 +1165,11 @@ def polygon2bbox(polygon):
     return np.min(array, axis=0).tolist() + np.max(array, axis=0).tolist()
 
 
-def xyxy2polygon(x1, y1, x2, y2):
+def xyxy2polygon(x1, y1=None, x2=None, y2=None):
+    if y1 is None and x2 is None and y2 is None:
+        bbox = x1
+        x1, y1, x2, y2 = bbox
+
     points = []
     points.append([x1, y1])
     points.append([x1, y2])
@@ -1179,24 +1328,24 @@ def connected_filter(image, min_area=0, w_h_ratio=None, largest_num=None, w_min=
         ratio_check = True
     for index in index_array:
         if index > 0:
+            sub_info = info[index]
             if ratio_check:
-                sub_info = info[index]
                 if min_ratio <= sub_info[2] / max(1, sub_info[3]) <= max_ratio:
                     pass
                 else:
                     continue
-                if w_min is not None:
-                    if sub_info[2] < w_min:
-                        continue
-                if w_max is not None:
-                    if sub_info[2] > w_max:
-                        continue
-                if h_min is not None:
-                    if sub_info[3] < h_min:
-                        continue
-                if h_max is not None:
-                    if sub_info[3] > h_max:
-                        continue
+            if w_min is not None:
+                if sub_info[2] < w_min:
+                    continue
+            if w_max is not None:
+                if sub_info[2] > w_max:
+                    continue
+            if h_min is not None:
+                if sub_info[3] < h_min:
+                    continue
+            if h_max is not None:
+                if sub_info[3] > h_max:
+                    continue
             mark_img[mark_img == index] = - index
     res_img = (mark_img < 0).astype('uint8') * final_value
     return res_img
@@ -1268,21 +1417,21 @@ def get_kernal(ksize, shape='rec'):
     return kernal
 
 
-def morphology_op(image, op, kernal_size, kermal_type=None):
+def morphology_op(image, op, kernal_size, kernal_type=None):
     """
-    op in ['open', 'close', 'eroid', 'dilate']
+    op in ['open', 'close', 'erode', 'dilate']
     ksize = [W, H]
-    kermal_type in ['rec', 'circle', 'cross']
+    kernal_type in ['rec', 'circle', 'cross']
     """
-    op_list = ['open', 'close', 'eroid', 'dilate']
+    op_list = ['open', 'close', 'erode', 'dilate']
     assert op in op_list, f"op {op} not in {op_list}"
     image = (image != 0).astype('uint8')
-    kernal = get_kernal(kernal_size, shape=kermal_type)
+    kernal = get_kernal(kernal_size, shape=kernal_type)
     if op == 'open':
         op_mark = cv2.MORPH_OPEN
     elif op == 'close':
         op_mark = cv2.MORPH_CLOSE
-    elif op == 'eroid':
+    elif op == 'erode':
         op_mark = cv2.MORPH_ERODE
     elif op == 'dilate':
         op_mark = cv2.MORPH_DILATE
@@ -1293,36 +1442,36 @@ def morphology_op(image, op, kernal_size, kermal_type=None):
     
 
 
-def eroid_op(image, kernal_size, kermal_type=None):
+def erode_op(image, kernal_size, kernal_type=None):
     """
     ksize = [W, H]
-    kermal_type in ['rec', 'circle', 'cross']
+    kernal_type in ['rec', 'circle', 'cross']
     """
-    return morphology_op(image, 'eroid', kernal_size, kermal_type)
+    return morphology_op(image, 'erode', kernal_size, kernal_type)
 
 
-def dilate_op(image, kernal_size, kermal_type=None):
+def dilate_op(image, kernal_size, kernal_type=None):
     """
     ksize = [W, H]
-    kermal_type in ['rec', 'circle', 'cross']
+    kernal_type in ['rec', 'circle', 'cross']
     """
-    return morphology_op(image, 'dilate', kernal_size, kermal_type)
+    return morphology_op(image, 'dilate', kernal_size, kernal_type)
 
 
-def open_op(image, kernal_size, kermal_type=None):
+def open_op(image, kernal_size, kernal_type=None):
     """
     ksize = [W, H]
-    kermal_type in ['rec', 'circle', 'cross']
+    kernal_type in ['rec', 'circle', 'cross']
     """
-    return morphology_op(image, 'open', kernal_size, kermal_type)
+    return morphology_op(image, 'open', kernal_size, kernal_type)
 
 
-def close_op(image, kernal_size, kermal_type=None):
+def close_op(image, kernal_size, kernal_type=None):
     """
     ksize = [W, H]
-    kermal_type in ['rec', 'circle', 'cross']
+    kernal_type in ['rec', 'circle', 'cross']
     """
-    return morphology_op(image, 'close', kernal_size, kermal_type)
+    return morphology_op(image, 'close', kernal_size, kernal_type)
 
 
 def image_center_paste(image, canvas):
@@ -1408,4 +1557,69 @@ def draw_hist(rgb_img):
         # print(hist.shape)
         plt.plot(hist, color=col)
         plt.xlim([0, 256])
-    plt.show()
\ No newline at end of file
+    plt.show()
+
+def gray_map_to_color_map(gray_map: np.ndarray, normalize: bool = True) -> np.ndarray:
+    """Compute color heatmap.
+
+    Args:
+        gray_map (np.ndarray): Final gray map computed by the distance metric.
+        normalize (bool, optional): Bool to normalize the gray map prior to applying
+            the color map. Defaults to True.
+
+    Returns:
+        np.ndarray: [description]
+    """
+    if normalize:
+        gray_map = (gray_map - gray_map.min()) / np.ptp(gray_map)
+    gray_map = gray_map * 255
+    gray_map = gray_map.astype(np.uint8)
+
+    gray_map = cv2.applyColorMap(gray_map, cv2.COLORMAP_JET)
+    gray_map = cv2.cvtColor(gray_map, cv2.COLOR_BGR2RGB)
+    return gray_map
+
+def superimpose_map(
+    image: np.ndarray, gray_map: np.ndarray, alpha: float = 0.4, gamma: int = 0, normalize: bool = False) -> np.ndarray:
+    """Superimpose gray map on top of in the input image.
+
+    Args:
+        gray_map (np.ndarray): gray map
+        image (np.ndarray): Input image
+        alpha (float, optional): Weight to overlay gray map
+            on the input image. Defaults to 0.4.
+        gamma (int, optional): Value to add to the blended image
+            to smooth the processing. Defaults to 0. Overall,
+            the formula to compute the blended image is
+            I' = (alpha*I1 + (1-alpha)*I2) + gamma
+        normalize: whether or not the gray maps should
+            be normalized to image min-max
+
+    Returns:
+        np.ndarray: Image with gray map superimposed on top of it.
+    """
+    gray_map = to_gray_image(gray_map)
+    if gray_map.shape[:2] != image.shape[:2]:
+        print(f"Warning: GrayMap shape {gray_map.shape[:2]} does not equal to Image shape {image.shape[:2]}")
+        H, W = image.shape[:2]
+        gray_map = image_resize(gray_map, [W, H], uint8=False)
+    gray_map = gray_map_to_color_map(gray_map.squeeze(), normalize=normalize)
+    superimposed_map = cv2.addWeighted(gray_map, alpha, image, (1 - alpha), gamma)
+    return superimposed_map
+
+def get_connect_components(img):
+    img = to_gray_image((img!= 0).astype('uint8'))
+    com_num, labels, stats, _ = cv2.connectedComponentsWithStats(img)
+    return com_num, labels, stats
+
+def mask_on_mask(input_mask, standard_mask):
+    input_mask_pro = to_gray_image(input_mask != 0)
+    standard_mask_pro = to_gray_image(standard_mask != 0)
+    assert input_mask_pro.shape == standard_mask_pro.shape
+    com_num, labels, stats = get_connect_components(input_mask_pro)
+    for index in range(com_num):
+        if index > 0:
+            if standard_mask_pro[labels == index].max() > 0:
+                labels[labels == index] = -1
+    final_mask = (labels < 0).astype('uint8')
+    return final_mask
\ No newline at end of file
