@@ -30,7 +30,6 @@ def cartesian_to_polar(point_xy, center_xy, reverse_y=True):
         raise RuntimeError(f'bad distance {r}')
     return r, theta
 
-
 def polar_to_cartesian(r, theta, center_xy, reverse_y=False):
     center_x, center_y = center_xy
     r = np.array(r)
@@ -44,6 +43,23 @@ def polar_to_cartesian(r, theta, center_xy, reverse_y=False):
     point_xy = np.vstack([x, y]).T.tolist()
     return point_xy
 
+def cal_length(vector):
+    return (np.array(vector) ** 2).sum() ** 0.5
+
+def get_angle_from_vector(vector, degree=False):
+    assert len(vector) == 2
+    assert (np.array(vector) ** 2).sum() > 0
+    length = cal_length(vector)
+    x, y = vector[:2]
+    sin_res = y / length
+    cos_res = x / length
+    radians = np.arccos(cos_res)
+    if sin_res < 0:
+        radians = np.pi * 2 - radians
+    if degree:
+        return radians / np.pi * 180
+    else:
+        return radians
 
 
 class LogicOp:
@@ -157,3 +173,58 @@ class LogicOp:
                 return False
         return func
 
+
+def make_rotate_matrix_2d(alpha, radian_mode=True):
+    """
+    x 轴正方向 - 3 点
+    y 轴正方向 - 12 点
+    顺时针旋转 alpha 角
+    """
+    if not radian_mode:
+        alpha = alpha / 180 * np.pi
+    matrix = np.array([
+        [np.cos(alpha), - np.sin(alpha)],
+        [np.sin(alpha), np.cos(alpha)]
+        ])
+    return matrix
+
+
+def point_rotate(point, center, alpha=0, rotate_matrix=None, radian_mode=True):
+    """_summary_
+
+    Args:
+        point ([x,y]): point to be rotated
+        center ([x,y]): center of rotation
+        alpha (float): rotate angle, if rotate_matrix set, this value will be ignored. Defaults to 0.
+        rotate_matrix (2d matrix): rotate matrix 2d, if not set, will build by angle. Defaults to None.
+        radian_mode (bool, optional): if True, alpha is considered to be a radian. Defaults to True.
+
+    Returns:
+        _type_: _description_
+    """
+    point = np.array(point)
+    center = np.array(center)
+    if (point == center).all():
+        return point
+    if rotate_matrix is None:
+        rotate_matrix = make_rotate_matrix_2d(alpha, radian_mode)
+    delt_x, delt_y = point - center
+    new_point = np.matmul(np.array([delt_x, delt_y]), rotate_matrix)
+    return new_point
+
+
+def polygon_rotate(polygon, center, alpha=0, rotate_matrix=None, radian_mode=True):
+    if len(polygon):
+        polygon_matrix = np.array(polygon) - center
+    else:
+        return polygon
+
+    assert polygon_matrix.ndim == 2
+    assert polygon_matrix.shape[1] == 2
+
+    if rotate_matrix is None:
+        rotate_matrix = make_rotate_matrix_2d(alpha, radian_mode)
+
+    rotated_polygon = np.matmul(polygon_matrix, rotate_matrix) + center
+
+    return rotated_polygon.tolist()
\ No newline at end of file
